@@ -1,78 +1,9 @@
 import ipywidgets as widgets
 import random as rd
 import numpy as np
+import sympy
 
-##########
-##########
-##########
-# (general) function to initiate the homework_notebook Student_ID request
-##########
-##########
-##########
 
-def generate_initiation_interface():
-    # generates a GUI with an 
-    # IntText inut for the STudent_ID,
-    # Button to start iniate_Params_HW(#Homeworknumber)() with
-    # output to print the values from iniate_Params_HW(#Homeworknumber)() in
-    StudID = widgets.IntText( description='Student ID:', value=0 )
-    button_StudID   = widgets.Button( description='Generate Parameters' )
-    output = widgets.Output()
-    
-    global vb
-    vb=widgets.VBox([StudID,button_StudID, output])
-    return
-
-##########
-##########
-##########
-# functions for compute_Params() for every homework_notebook
-##########
-##########
-##########
-
-def compute_Params_HW3(b):
-    # diable Button from initatl GUI
-    # create hw3 in global namespace
-    # store homework_three instance in hw3
-    # print in output widget of inital GUI
-    vb.children[1].disabled = True
-    global hw3
-    hw3 = homework_three(vb.children[0].value)
-    with vb.children[2]:
-        print("##########\n\nParameters for Question 1:\n\n"\
-        "Your glucose yield Y_(XS) is {} C-mol glucose/C-mol biomass\n\n"\
-        "Your biomass composition is CH_({})O_({})N_({})\n\n##########".format(hw3.newYxs,hw3.newH,hw3.newO,hw3.newN))
-        print("##########\n\nParameters for Question 2:\n\n"\
-        "Your O_2 Exchange-Rate is {} mol/h\n\n"\
-        "Your CO_2 Exchange-Rate is {} mol/h\n\n##########".format(hw3.newRO2,hw3.newRCO2))
-
-#####
-#####
-#####
-
-def compute_Params_HW4(b):
-    # diable Button from initatl GUI
-    # create hw4 in global namespace
-    # store homework_three instance in hw3
-    # print in output widget of inital GUI
-    vb.children[1].disabled = True
-    global hw4
-    hw4 = homework_four(vb.children[0].value)
-    with vb.children[2]:
-        #NOTE: Chemical Compositions are, as of yet, not actually modified in HW4 
-        #      but are treated like it for the sake of the students
-        print("##########\n\nParameters for Question 1:\n\n"\
-        "Approximate Chemical Compostion of Fats: CH_(1.92)O_(0.12)\n" \
-        "Approximate Chemical Compostion of Proteins: CH_(1.57)O_(0.32)N_(0.26)\n\n" \
-        "Heat of combustion for protein: {} kcal/g,\n" \
-        "Heat of combustion for Carbohydrates: {} kcal/g,\n" \
-        "Heat of combustion for Fat: {} kcal/g\n\n##########".format(hw4.newCoP,hw4.newCoC,hw4.newCoF))
-        print("##########\n\nParameters for Question 3:\n\n"\
-        "Oxygen consumption rate: {} mol/h,\n" \
-        "Carbon dioxide production rate: {} mol/h,\n" \
-        "Nitrogen secretion rate: {} g N/h\n\n##########".format(hw4.newOCR,hw4.newCPR, hw4.newNSR))
-        
 ##########
 ##########
 ##########
@@ -1430,9 +1361,315 @@ Task is determine the metabolic rate Rxq [kcal/h]\n
 (* The Yied of CO_2 (Y_cc) = 1 [mol/cmol])\n
 (* Don’t forget to calculate Ycn out of the nitrogen secretion rate!)\n
  * Next, enter the yields into the three equations from [Question 2].\n
- * Calculate Yxq [kJ/cmol] (p. 36-38  in the QM3-script ‘Quantitative Microbiologie week 4 – Folien’\n
+ * Calculate Yxq [kcal/cmol] (p. 36-38  in the QM3-script ‘Quantitative Microbiologie week 4 – Folien’\n
  * The formula for Rxq is Yxq [kcal/cmol] * r_co2 [cmol/h]\n
 ##########\n
             """
 
+            print(txt)
+
+##########
+##########
+##########
+#class for homework 5
+##########
+##########
+##########
+
+class homework_five():
+#class homework_five_test():
+    def __init__(self):
+        self.init_display = self.build_init_display()
+        
+        
+        
+    def start_homework(self, stdID):
+        # Check if Student-ID is an integer and if the function is to be run in "test-mode"
+        
+        #'TEst-Mode' is activated by the Std-ID input: 123456789
+        # For this to be viable, 123456789 CANNEVER be a viable student-id
+        if type(stdID) == int and stdID == 123456789:
+            self.stdID = "testing"
+        elif type(stdID) == int and stdID != 123456789:
+            self.stdID = stdID
+        else:
+            print("Student ID required to be an integer.\nPlease try again!\n\n")
+            pass
+        
+        # How many rngs are to be created for new/changed parameters
+        self.numberofrandomnumbers = 10
+        
+        # Create RNs on the basis of the studentID
+        self.initiate_homework()
+        
+        # Create variables using the generated RNs
+        self.create_question_variables()
+        
+        # List of self.parameters:
+        #self.stID
+        #self.listofrndms
+        
+        #self.v1
+        #self.v4
+        #self.v5
+        #self.v6
+        #self.v7
+        
+        self.hq1tgl = 0 #help_question_1_toggle
+        self.hq2tgl = 0 #help_question_2_toggle
+        
+        # build widgets for the questions
+        self.question_one_display = self.build_question_display()
+        self.question_two_display = self.build_question_display()
+        
+        with self.init_display.children[2]:
+            txt= """
+##########\n\nParameters for Question 2:\n\n
+Flux v1 = {}\n
+Flux v4 = {}\n
+Flux v5 = {}\n
+Flux v6 = {}\n
+Flux v7 = {}\n\n
+##########
+            """.format(hw5.v1,hw5.v4,hw5.v5,hw5.v6,hw5.v7)
+            print(txt)
+        
+        
+    def initiate_homework(self):
+    # Parameters:
+    # stdID is an integer used to seed the rng
+        # IF stdID is given the string "test", it's supposed to return the string "test"
+        # This string is then used to thest functions using standard values
+    # counter is an integer used to define how many rngs are created.
+        #print("Matriculation-Number accepted.\n\nGenerating random numbers..")
+        #print("Matriculation-Number accepted.\n\nGenerating personalized homework parameters..")
+        if self.stdID == "testing":
+            print("#####\n\nTesting functionality\n\n#####")
+            print("#####\n\The Std-ID is not a valid Student-ID\n and CAN'T generate a valid code!\n\n#####")
+            # NOTE: no self.listofrndms is created in test mode
+        else:
+        #RNG for this homework
+        #IMPORTANT: rd.seed() is set GLOBALLY, as far as we observed
+        #this means: the solution-codes are ALWAYS IDENTICAL FOR A UNIQUE STid
+        #if rd.seed() is not refreshed after generation of listofrndms.
+        #In this version, the generated solution-codes are supposed to be random, therefore. rd.seed() gets refreshed
+            rd.seed(self.stdID)
+            self.listofrndms = [rd.random() for i in range(self.numberofrandomnumbers)] #create new random numbers that can be used to modify parameters
+            rd.seed()
+        return
+    
+    def create_question_variables(self):
+        #function returns all generated homework parameters and a boolean 
+        #thats used to toggls help_question_1_toggle to  0 (helpoption was not used)
+        if self.stdID != "testing":
+            #print("\nGenerating personalized homework parameters...\n")
+            
+            ###PARAMATERS for QUESTION 2           
+            ### STEP 1: Create modified parameters
+            self.v1 = 19+round(self.listofrndms[0]*2,1) #range 19.0-21.0
+            self.v4 = 30+round(self.listofrndms[1]*2,1) #range 30.0-32.0
+            self.v5 = 9+round(self.listofrndms[2]*1,1) #range 9.0-10.0
+            self.v6 = 4.5+round(self.listofrndms[3]*1,1) #range 4.5.0-5.5
+            self.v7 = 4.5+round(self.listofrndms[4]*1,1) #range 4.5.0-5.5 
+            
+            ### STEP 2: Determine if the Equations are consistent or not:
+            ### MOVED TO QUESTION-2-SOLVER
+                
+        
+        elif self.stdID == "testing":
+            ###PARAMATERS for QUESTION 2
+            self.v1 = 19.3
+            self.v4 = 31.1
+            self.v5 = 9.5
+            self.v6 = 5.5
+            self.v7 = 5.0
+            
+        return
+            
+###############
+###############
+###############
+    def test_homework_five(self, litmatr, test):
+        testnumber = "123456789"
+        #Parameters:
+        #listofbalances: Student solutions. Will be comparied with opimal solution and decides if task was done correctly or not
+        #studentindep = Student solution part 2
+        
+        #NO parameters were changed for this question
+        #hq1tgl: boolean. Decides if the student used the help-option or not
+
+        #output formula:
+        # X-YZ
+        # X: was the help used? [yes] (randrange(5,9) or [no] randrange(0,4)
+        #    -> randomint
+        # YZ: Summ(studentID[0]+studentID[1]) [<10] randrange(0,4)) OR [>= 10] randrange(5,9)):
+        #    -> 2x randomint
+        if test != testnumber:
+            return
+        
+        print("\n####\nQuestion 1\n#####\n")
+        
+        if litmatr == None:
+        
+            litmatr = [[1,-1,0,0,0,0,0,0],
+            [-1,2,-1,0,0,0,0,0],
+            [1,0,1,-1,-1,0,0,0],
+            [0,0,0,0,1,-1,-1,0],
+            [0,1,1,0,0,1,0,-1],
+            [0,2,0,-1,0,0,-2,0]]
+        
+        #Calculate the independents of litmatr
+        _, litinds = sympy.Matrix(litmatr).T.rref()
+        print("Number of independent reactions in matrix:",len(litinds))
+        print("Position of independent reactions in matrix:",litinds)
+        
+        print("\n####\nQuestion 2\n#####\n")
+        
+
+        concounter = 0
+        noncon = []
+
+        # background check relations:
+        # v1 = v2 = v3 = (0.5v7 + 0.5v8)
+        # 0.5v5 = v6 = v7
+
+        # v5 - v6 - v7 = 0 --> 2v6 - v6 - v7 = 0 --> v6 - v7 = 0
+        # v5 - v6 - v7 = 0 --> v5 - 2v6/2v7 = 0
+        # 2v2 - v4 - 2v6 --> 2v1 - v4 -2v6/2v7 = 0
+
+        # => 2v1 - v4 = 2v6/2v7 = v5 with v6 - v7 = 0
+        p1 = [2*self.v1 - self.v4, 2*round(self.v1*0.05,2) + round(self.v4*0.05,2),"v1, v4"]
+        p2 = [self.v5, round(self.v5*0.05,2),"v5"]
+        p3 = [2*self.v6, round(2*self.v6*0.05,2),"v6"]
+        p4 = [2*self.v7, 2*round(self.v7*0.05,2),"v7"]
+
+        # Check 1
+        if (self.v6 - self.v7) + (self.v6*0.05 + self.v7*0.05) >= 0 > (self.v6 - self.v7) - (self.v6*0.05 + self.v7*0.05) or (self.v7 - self.v6) + (self.v6*0.05 + self.v7*0.05) >= 0 > (self.v7 - self.v6) - (self.v6*0.05 + self.v7*0.05):
+            concounter += 0            
+        else:
+            concounter += 1
+            noncon.append([p3,p4])
+
+        #Check 2 - 4
+        if (p1[0]-p2[0])+(p1[1]+p2[1]) >= 0 > (p1[0]-p2[0])-(p1[1]+p2[1]) or (p2[0]-p1[0])+(p2[1]+p1[1]) >= 0 > (p2[0]-p1[0])-(p2[1]+p1[1]):
+            concounter += 0
+        else:
+            concounter += 1
+            noncon.append([p1,p2])
+        if (p1[0]-p3[0])+(p1[1]+p3[1]) >= 0 > (p1[0]-p3[0])-(p1[1]+p3[1]) or (p3[0]-p1[0])+(p3[1]+p1[1]) >= 0 > (p3[0]-p1[0])-(p3[1]+p1[1]):
+            concounter += 0
+        else:
+            concounter += 1
+            noncon.append([p1,p3])
+        if (p2[0]-p3[0])+(p2[1]+p3[1]) >= 0 > (p2[0]-p3[0])-(p2[1]+p3[1]) or (p3[0]-p2[0])+(p3[1]+p2[1]) >= 0 > (p3[0]-p2[0])-(p3[1]+p2[1]):
+            concounter += 0
+        else:
+            concounter += 1
+            noncon.append([p2,p3])
+        if (p1[0]-p4[0])+(p1[1]+p4[1]) >= 0 > (p1[0]-p4[0])-(p1[1]+p4[1]) or (p4[0]-p1[0])+(p4[1]+p1[1]) >= 0 > (p4[0]-p1[0])-(p4[1]+p1[1]):
+            concounter += 0
+        else:
+            concounter += 1
+            noncon.append([p1,p4])
+        if (p2[0]-p4[0])+(p2[1]+p4[1]) >= 0 > (p2[0]-p4[0])-(p2[1]+p4[1]) or (p4[0]-p2[0])+(p4[1]+p2[1]) >= 0 > (p4[0]-p2[0])-(p4[1]+p2[1]):
+            concounter += 0
+        else:
+            concounter += 1
+            noncon.append([p2,p4])
+
+    #Calculation of the solution:
+        print("Concounter=",concounter)
+        print("Glucose/v1",self.v1)
+        print("Lactate/v4",self.v4)
+        print("v5",self.v5)
+        print("v6",self.v6)
+        print("v7",self.v7)
+        print("Nonconform pairs:",noncon)
+        print("2v1-v4-v5 = 0 | {} +- {}".format((p1[0]-p2[0]),(p1[1]+p2[1])))
+        print("2v1-v4-2v6 = 0 | {} +- {}".format((p1[0]-p3[0]),(p1[1]+p3[1])))
+        print("v5-2v6 = 0 | {} +- {}".format((p2[0]-p3[0]),(p2[1]+p3[1])))
+        print("2v1-v4-2v7 = 0 | {} +- {}".format((p1[0]-p4[0]),(p1[1]+p4[1])))
+        print("v5-2v7 = 0 | {} +- {}".format((p2[0]-p4[0]),(p2[1]+p4[1])))
+        print("v6-v7 = 0 | {} +- {}".format((self.v6 - self.v7),(self.v6*0.05 + self.v7*0.05)))    
+        #return
+
+             
+    ######
+    # functions that create widgets
+    #
+    #
+    #
+    ###
+    
+    def build_init_display(self):
+        StudID = widgets.IntText( description='Student ID:', value=0 )
+        button_StudID   = widgets.Button( description='Generate Parameters' )
+        output = widgets.Output()
+
+        disp=widgets.VBox([StudID,button_StudID, output]) 
+        return disp
+    
+###############
+###############
+###############
+    def build_question_display(self):
+            listofwidgets = []
+
+            # build question label/output (top 0)
+            # build 'Get help button' (top 1)
+            # build output (for commentsof help button) (top 2)
+            listofwidgets.append(widgets.Output())
+
+            listofwidgets.append(widgets.Button(
+                value=False,
+                description='Calculation Help',
+                disabled=False,
+                button_style='', # 'success', 'info', 'warning', 'danger' or ''
+                tooltip='Click to get a tipp',
+            #     icon='check' # (FontAwesome names without the `fa-` prefix)
+            ))
+
+            listofwidgets.append(widgets.Output())
+
+            disp = widgets.VBox(listofwidgets)
+            return disp
+    
+    ######
+    # ON-CLICK functions
+    # 2 per question
+    #
+    #
+    ###
+    def on_init_homework_button_clicked(self, click):
+        self.init_display.children[1].disabled = True
+        StdID = self.init_display.children[0].value
+        self.start_homework(StdID)
+    
+    def on_question_one_help_button_clicked(self, click):
+        self.hq1tgl = 1
+        self.question_one_display.children[1].disabled = True
+        with self.question_one_display.children[-1]:
+            txt = """
+##########\n
+The task is to establish the independent balances of the stoichiometric matrix G:\n\n
+* First step is to establish all balances for the six intracellular metabolites.\nThis is also described in the scripts 'Stoichiometric models I' p. 18.\n\n
+* Second step is the gaussian elimination of the converted matrix\nThis is also described in the scripts 'Stoichiometric models I' p. 20-21.\n\n
+The number of independent balances can be read off from there.\n
+##########\n
+            """#HHAVE TO HAVE ANOTHE RLOOK AT THIS
+            print(txt)
+        
+    def on_question_two_help_button_clicked(self, click):
+        self.hq2tgl = 1
+        self.question_two_display.children[1].disabled = True
+        with self.question_two_display.children[-1]:
+            txt = """
+##########\n
+This task is to determine if the observed flux values are consistent or not.\nFor this scenario, consistency is proven when all established dependent relationships\nare consistent with the observed metabolic flux-rates\n\n
+* Calculate the standard deviation for all experimental flux rates
+* In Question 1, you have calculated the interdepencies of the flux equations of the stoichiometric matrix\nuse these established relationships between fluxes to determine\nif they are consistent to each other.
+* If you find a pair of dependent fluxes that are not consistent to one another, the system itself can’t be consistent.\n
+Example:\nv_Acetate - v_Ethanol = 0 |  (5.0 - 5.5) +/- (0.25 + 0.275) = 0\n 
+##########\n
+            """#HHAVE TO HAVE ANOTHE RLOOK AT THIS
             print(txt)
\ No newline at end of file
